/*
 * $Id: getip.c,v 1.1 2024-10-04 08:02:35+05:30 Cprogrammer Exp mbhangui $
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>
#include <byte.h>
#include <stralloc.h>
#include <strerr.h>
#include "haveip6.h"

int
getip(const char *host, stralloc *insa)
{
	int                 i;
#if defined(LIBC_HAS_IP6) && defined(IPV6)
	char                addrBuf[INET6_ADDRSTRLEN];
	struct addrinfo hints = {0}, *addr_res = 0, *addr_res0 = 0;
	struct sockaddr     sa;
	struct in_addr      addr4;
	struct in6_addr     addr6;
	struct sockaddr_in *in4 = (struct sockaddr_in *) &sa;
	struct sockaddr_in6 *in6 = (struct sockaddr_in6 *) &sa;
#else
	struct hostent *hp;
#endif

	insa->len = 0;
#if defined(LIBC_HAS_IP6) && defined(IPV6)
	if ((i = inet_pton(AF_INET, host, &addr4)) == 1) {
		if (!stralloc_copys(insa, host))
			strerr_die1x(111, "tcpclient: fatal: out of memory");
		return 0;
	} else
	if (i == -1)
		strerr_die1x(111, "tcpclient: fatal: invalid address family AF_INET");
	if ((i = inet_pton(AF_INET6, host, &addr6)) == 1) {
		if (!stralloc_copys(insa, host))
			strerr_die1x(111, "tcpclient: fatal: out of memory");
		return 0;
	} else
	if (i == -1)
		strerr_die1x(111, "tcpclient: fatal: invalid address family AF_INET6");
	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;
	hints.ai_flags = AI_PASSIVE;    /* For wildcard IP address */
	hints.ai_protocol = 0;          /* Any protocol */
	hints.ai_canonname = 0;
	hints.ai_addr = 0;
	hints.ai_next = 0;
	if ((i = getaddrinfo(host, 0, &hints, &addr_res0)))
		strerr_die2x(111, "tcpclient: fatal: getaddrinfo: ", gai_strerror(i));
	for (addr_res = addr_res0; addr_res; addr_res = addr_res->ai_next) {
		byte_copy((char *) &sa, addr_res->ai_addrlen, (char *) addr_res->ai_addr);
		if (sa.sa_family == AF_INET) {
			in4 = (struct sockaddr_in *) &sa;
			if (!inet_ntop(AF_INET, (void *) &in4->sin_addr, addrBuf, INET_ADDRSTRLEN)) {
				freeaddrinfo(addr_res0);
				strerr_die1sys(111, "tcpclient: fatal: inet_ntop: ");
			}
			if (!stralloc_copys(insa, addrBuf)) {
				freeaddrinfo(addr_res0);
				strerr_die1x(111, "tcpclient: fatal: out of memory");
			}
			freeaddrinfo(addr_res0);
			return 0;
		} else
		if (sa.sa_family == AF_INET6) {
			in6 = (struct sockaddr_in6 *) &sa;
			if (!inet_ntop(AF_INET6, (void *) &in6->sin6_addr, addrBuf, INET_ADDRSTRLEN)) {
				freeaddrinfo(addr_res0);
				strerr_die1sys(111, "tcpclient: fatal: inet_ntop: ");
			}
			if (!stralloc_copys(insa, addrBuf)) {
				freeaddrinfo(addr_res0);
				strerr_die1x(111, "tcpclient: fatal: out of memory");
			}
			freeaddrinfo(addr_res0);
			return 0;
		} else
			continue;
	}
	freeaddrinfo(addr_res0);
	strerr_die1x(111, "tcpclient: fatal: no usable address found");
#else
	if (inet_addr(host) != INADDR_NONE) {
		if (!stralloc_copys(insa, host))
			strerr_die1x(111, "tcpclient: fatal: out of memory");
		return 0;
	}
	if (!(hp = gethostbyname(host)))
		strerr_die2sys(111, "tcpclient: fatal: gethostbyname: ", hstrerror(h_errno));
	if (!stralloc_copys(insa, inet_ntoa(*((struct in_addr *) hp->h_addr_list[0]))))
		strerr_die1x(111, "tcpclient: fatal: out of memory");
	return 0;
#endif
}

/*
 * $Log: getip.c,v $
 * Revision 1.1  2024-10-04 08:02:35+05:30  Cprogrammer
 * Initial revision
 *
 */
